% PURPOSE:
%        Estimates a multivariate GARCH model using the DCC estimator of Engle and Sheppard
% 
% USAGE:
%        [parameters, loglikelihood, Ht, Rt, Qt, likelihoods, stdresid, stderrors, A,B, jointscores]...
%                =dcc_mvgarch(data,dccQ,dccP,archQ,garchP)
% 
% INPUTS:
%      data          = A zero mean t by k (asset number) vector of residuals from some filtration [t by k]
%      dccQ          = The lag length of the innovation term in the DCC estimator (a scalar)
%      dccP          = The lag length of the lagged correlation matrices in the DCC estimator (a scalar)
%      archQ         = One of two things:     A scalar, q     in which case a p innovation model is estimated for each series
%                                             A k by 1 vector in which case the ith series has innovation terms p=archP(i)
%      garchP        = One of two things:     A scalar, p     in which case a q GARCH lags is used in estimation for each series
%                                             A k by 1 vector in which case the ith series has lagged variance terms q=archQ(i)
%
% OUTPUTS:
%      parameters    = A vector of parameters estimated form the model of the form
%                          [GarchParams(1) GarchParams(2) ... GarchParams(k) DCCParams]
%                          where the Garch Parameters from each estimation are of the form
%                          [omega(i) alpha(i1) alpha(i2) ... alpha(ip(i)) beta(i1) beta(i2) ... beta(iq(i))]
%      loglikelihood = The log likelihood evaluated at the optimum
%      Ht            = A k by k by t array of conditional variances
%      Rt            = A k by k by t array of Rt elements
%      Qt            = A k by k by t array of Qt elements
%      stdresid      = A [t by k] matrix of standardized residuals
%      likelihoods   = the estimated likelihoods t by 1
%      stderrors     = A length(parameters)^2 matrix of estimated correct standard errors
%      A             = The estimated A from the robust standard errors
%      B             = The estimated B from the standard errors
%      jointscores   = The estimated scores of the likelihood t by length(parameters)
%      H             = Conditional Volatility univariate
% 
% COMMENTS:
%
% Modifications: Sylvain Benoit    Date Revision: 11/10/2011
% Initial codes Author: Kevin Sheppard
% kevin.sheppard@economics.ox.ac.uk
% Revision: 3    Date: 4/1/2004

function [parameters, loglikelihood, Rt, Qt, stdresid, likelihoods]=dcc_part(data,dccQ,dccP,archQ,garchP,stdresid,parameters)
%function [parameters, loglikelihood, Ht, Rt, Qt, stdresid, likelihoods]=dcc_part(data,dccQ,dccP,archQ,garchP,stdresid,parameters,HH)

%% Now lest do the multivariate garching using dcc_mvgarch_likelihood

options  =  optimset('fmincon');
options  =  optimset(options , 'Display'            , 'on');
options  =  optimset(options , 'Diagnostics'        , 'off');
options  =  optimset(options , 'LevenbergMarquardt' , 'on');
options  =  optimset(options , 'LargeScale'         , 'off');

dccstarting=[ones(1,dccQ)*.01/dccQ ones(1,dccP)*.97/dccP];

disp(' ')
disp(' === Second Step : Estimation of the DCC ===')
disp(' ')

[dccparameters,dccllf,EXITFLAG,OUTPUT,LAMBDA,GRAD]=fmincon('dcc_mvgarch_likelihood',dccstarting, ones(size(dccstarting)),[1-2*options.TolCon],[],[],...
    zeros(size(dccstarting))+2*options.TolCon,[],[],options,stdresid,dccQ,dccP);


%% We now have all of the estimated parameters

%parameters=[];
%H=zeros(t,k);
%for i=1:k
%    parameters=[parameters;univariate{i}.parameters];
%    H(:,i)=univariate{i}.ht;
%end
parameters=[parameters;dccparameters'];

[t,k]=size(data);

if length(archQ)==1
    archQ=ones(1,k)*archQ;
end

if length(garchP)==1
    garchP=ones(1,k)*garchP;
end
%% We now have Ht and the likelihood but we want the FULL LIKELIHOOD

% options  =  optimset('fminsearch');
% options  =  optimset(options , 'Display'            , 'iter');
% options  =  optimset(options , 'Diagnostics'        , 'on');
% options  =  optimset(options , 'LevenbergMarquardt' , 'on');
% options  =  optimset(options , 'LargeScale'         , 'off');
% options  =  optimset(options , 'MaxIter'            , 1);
% % ? utiliser avec parcimonie car on ne ma?trise plus les contraintes!
% [parameters,loglikelihood,exitflag]=fminsearch('dcc_mvgarch_full_likelihood', parameters, options, data, archQ, garchP, dccQ, dccP);
% [loglikelihood, Rt, likelihoods, Qt]=dcc_mvgarch_full_likelihood(parameters, data, archQ, garchP, dccQ, dccP);

[loglikelihood, Rt, likelihoods, Qt]=dcc_mvgarch_full_likelihood(parameters, data, archQ, garchP, dccQ, dccP);


likelihoods=-likelihoods;                                                   %output;
loglikelihood=-loglikelihood;                                               %output;
%{
Ht=zeros(k,k,t);
Hstd=H.^(0.5);
for i=1:t
    Ht(:,:,i)=diag(Hstd(i,:))*Rt(:,:,i)*diag(Hstd(i,:));
end
save tempHt Ht
clear Ht
%}